==> TundereBird-0.24.tar.bz2: TundereBird-0.24/core/Smdn.Net.Imap4.Client/Smdn.Net.Imap4.Client.Transaction.BuiltIn/FetchTransaction.cs <==
// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2008-2010 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Collections;
using Smdn.Net.Imap4.Protocol;
using Smdn.Net.Imap4.Protocol.Client;

namespace Smdn.Net.Imap4.Client.Transaction.BuiltIn {
  internal class FetchTransaction : ImapTransactionBase<ImapTransactionResult<ImapMailboxMessage[]>>, IImapExtension {
    ImapCapability IImapExtension.RequiredCapability {
      get { return fetchModifiersCapabilityRequirement; }
    }

    public FetchTransaction(ImapConnection connection, bool uid, ImapCapability fetchModifiersCapabilityRequirement)
      : base(connection)
    {
      this.uid = uid;
      this.fetchModifiersCapabilityRequirement = fetchModifiersCapabilityRequirement;
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Request.Arguments.ContainsKey("sequence set"))
        return ProcessArgumentNotSetted;
      else if (!Request.Arguments.ContainsKey("message data item names or macro"))
        return ProcessArgumentNotSetted;
      else
        return ProcessFetch;
    }

    private void ProcessArgumentNotSetted()
    {
      FinishError(ImapTransactionResultCode.RequestError, "arguments 'sequence set' and 'message data item names or macro' must be setted");
    }

    // RFC 4466 - Collected Extensions to IMAP4 ABNF
    // http://tools.ietf.org/html/rfc4466
    // 2.4. Extensions to FETCH and UID FETCH Commands
    //    Arguments:  sequence set
    //                message data item names or macro
    //                OPTIONAL fetch modifiers
    //    Responses:  untagged responses: FETCH
    //    Result:     OK - fetch completed
    //                NO - fetch error: cannot fetch that data
    //                BAD - command unknown or arguments invalid
    private void ProcessFetch()
    {
      // FETCH / UID FETCH
      if (Request.Arguments.ContainsKey("fetch modifiers"))
        SendCommand(uid ? "UID FETCH" : "FETCH",
                    ProcessReceiveResponse,
                    Request.Arguments["sequence set"],
                    Request.Arguments["message data item names or macro"],
                    Request.Arguments["fetch modifiers"]);
      else
        SendCommand(uid ? "UID FETCH" : "FETCH",
                    ProcessReceiveResponse,
                    Request.Arguments["sequence set"],
                    Request.Arguments["message data item names or macro"]);
    }

    protected override void OnDataResponseReceived(ImapDataResponse data)
    {
      if (data.Type == ImapDataResponseType.Fetch)
        ImapDataResponseConverter.FromFetchInternal(messages, data);

      base.OnDataResponseReceived(data);
    }

    protected override void OnTaggedStatusResponseReceived(ImapTaggedStatusResponse tagged)
    {
      if (tagged.Condition == ImapResponseCondition.Ok) {
        Finish(new ImapTransactionResult<ImapMailboxMessage[]>(messages.Values.ToArray(), tagged.ResponseText.Text));
        messages.Clear();
      }
      else {
        base.OnTaggedStatusResponseReceived(tagged);
      }
    }

    private readonly bool uid;
    private /*readonly*/ ImapCapability fetchModifiersCapabilityRequirement;
    private /*readonly*/ Dictionary<long, ImapMailboxMessage> messages = new Dictionary<long, ImapMailboxMessage>();
  }
}

==> TundereBird-0.28.tar.bz2: TundereBird-0.28/core/Smdn.Net.Imap4.Client/Smdn.Net.Imap4.Client.Transaction.BuiltIn/FetchTransaction.cs <==
// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2008-2010 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Collections;
using Smdn.Net.Imap4.Protocol;
using Smdn.Net.Imap4.Protocol.Client;

namespace Smdn.Net.Imap4.Client.Transaction.BuiltIn {
  internal class FetchTransaction<TMessageAttribute> :
    ImapTransactionBase<ImapTransactionResult<TMessageAttribute[]>>,
    IImapExtension
    where TMessageAttribute : ImapMessageAttributeBase
  {
    ImapCapability IImapExtension.RequiredCapability {
      get { return fetchModifiersCapabilityRequirement; }
    }

    public FetchTransaction(ImapConnection connection, bool uid, ImapCapability fetchModifiersCapabilityRequirement)
      : base(connection)
    {
      this.uid = uid;
      this.fetchModifiersCapabilityRequirement = fetchModifiersCapabilityRequirement;
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (!Request.Arguments.ContainsKey("sequence set"))
        return ProcessArgumentNotSetted;
      else if (!Request.Arguments.ContainsKey("message data item names or macro"))
        return ProcessArgumentNotSetted;
      else
        return ProcessFetch;
    }

    private void ProcessArgumentNotSetted()
    {
      FinishError(ImapTransactionResultCode.RequestError, "arguments 'sequence set' and 'message data item names or macro' must be setted");
    }

    // RFC 4466 - Collected Extensions to IMAP4 ABNF
    // http://tools.ietf.org/html/rfc4466
    // 2.4. Extensions to FETCH and UID FETCH Commands
    //    Arguments:  sequence set
    //                message data item names or macro
    //                OPTIONAL fetch modifiers
    //    Responses:  untagged responses: FETCH
    //    Result:     OK - fetch completed
    //                NO - fetch error: cannot fetch that data
    //                BAD - command unknown or arguments invalid
    private void ProcessFetch()
    {
      // FETCH / UID FETCH
      if (Request.Arguments.ContainsKey("fetch modifiers"))
        SendCommand(uid ? "UID FETCH" : "FETCH",
                    ProcessReceiveResponse,
                    Request.Arguments["sequence set"],
                    Request.Arguments["message data item names or macro"],
                    Request.Arguments["fetch modifiers"]);
      else
        SendCommand(uid ? "UID FETCH" : "FETCH",
                    ProcessReceiveResponse,
                    Request.Arguments["sequence set"],
                    Request.Arguments["message data item names or macro"]);
    }

    protected override void OnDataResponseReceived(ImapDataResponse data)
    {
      if (data.Type == ImapDataResponseType.Fetch)
        ImapDataResponseConverter.FromFetch(messages, data);

      base.OnDataResponseReceived(data);
    }

    protected override void OnTaggedStatusResponseReceived(ImapTaggedStatusResponse tagged)
    {
      if (tagged.Condition == ImapResponseCondition.Ok) {
        Finish(new ImapTransactionResult<TMessageAttribute[]>(messages.Values.ToArray(), tagged.ResponseText));
        messages.Clear();
      }
      else {
        base.OnTaggedStatusResponseReceived(tagged);
      }
    }

    private readonly bool uid;
    private /*readonly*/ ImapCapability fetchModifiersCapabilityRequirement;
    private /*readonly*/ Dictionary<long, TMessageAttribute> messages = new Dictionary<long, TMessageAttribute>();
  }
}
